<?php
/*************************
  Coppermine Photo Gallery
  ************************
  Copyright (c) 2003-2006 Coppermine Dev Team
  v1.1 originally written by Gregory DEMAR

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
**********************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

require_once 'bridge/udb_base.inc.php';

class cpg_udb extends core_udb {

	function cpg_udb()
	{
		global $BRIDGE, $mainframe;
		
		$this->path = $BRIDGE['relative_path_to_config_file'];
		$this->boardurl = $BRIDGE['full_forum_url'];
		$this->use_post_based_groups = $BRIDGE['use_post_based_groups'];
		$this->multigroups = 0;
		$this->group_overrride = 1;

		define( '_JEXEC', 1);
		define('JPATH_BASE', $this->path);
		
		require_once ( JPATH_BASE .'/includes/defines.php' );
		require_once ( JPATH_BASE .'/includes/application.php' );

		$mainframe = new JSite();
		$mainframe->setConfiguration(JPATH_CONFIGURATION . DS . 'configuration.php');
		$mainframe->setSession($this->boardurl . $mainframe->getClientId());
		
		$this->_user =& $mainframe->getUser();	
		$this->_mainframe =& $mainframe;
		
		$registry =& JFactory::getConfig();
	
		// Database connection settings
		$this->db = array(
			'name' => $registry->getValue('config.db'),
			'host' => $registry->getValue('config.host'),
			'user' => $registry->getValue('config.user'),
			'password' => $registry->getValue('config.password'),
			'prefix' => $registry->getValue('config.dbprefix')
		);

		// Board table names
		$this->table = array(
			'users' => 'users',
			'groups' => 'core_acl_aro_groups'
		);

		// Derived full table names
		$this->usertable = '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['users'];
		$this->groupstable =  '`' . $this->db['name'] . '`.' . $this->db['prefix'] . $this->table['groups'];
		
		// Table field names
		$this->field = array(
			'username' => 'name', // name of 'username' field in users table
			'user_id' => 'id', // name of 'id' field in users table
			'password' => 'password', // name of 'password' field in users table
			'email' => 'email', // name of 'email' field in users table
			'regdate' => 'UNIX_TIMESTAMP(registerDate)', // name of 'registered' field in users table
			'lastvisit' => 'UNIX_TIMESTAMP(lastvisitDate)', // last time user logged in
			'active' => "IF(block, 'NO', 'YES')", // is user account active?
			'location' => "''", // name of 'location' field in users table
			'website' => "''", // name of 'website' field in users table
			'usertbl_group_id' => 'gid', // name of 'group id' field in users table
			'grouptbl_group_id' => 'id', // name of 'group id' field in groups table
			'grouptbl_group_name' => 'name' // name of 'group name' field in groups table
		);
		
		// Pages to redirect to
		$this->page = array(
			'register' => '/index.php?option=com_registration&task=register',
			'editusers' => '/administrator/index.php',
			'edituserprofile' => '/index.php?option=com_user&task=UserDetails&Itemid=20',
		);

		// Group ids - admin and guest only.
		$this->admingroups = array(24,25);
		$this->guestgroup = 3;
		
		// Connect to db
		$db = $registry->getValue('config.dbtype') == 'mysql' ? $mainframe->getDBO() : false;
		$this->connect($db ? $db->_resource : false);
	}

	function authenticate(){

		global $USER_DATA;

		if (!$this->_user->get('id')) {
			$this->load_guest_data();
		} else {
			$USER_DATA['user_id'] = $this->_user->get('id');
			$USER_DATA['user_name'] = $this->_user->get('name');
			$USER_DATA['groups'] = array($this->_user->get('gid') + ($this->use_post_based_groups ? 100 : 0));
		}

		$user_group_set = '(' . implode(',', $USER_DATA['groups']) . ')';
		$USER_DATA = array_merge($USER_DATA, $this->get_user_data($USER_DATA['groups'][0], $USER_DATA['groups'], $this->guestgroup));
		$USER_DATA['has_admin_access'] = (in_array($USER_DATA['groups'][0] - ($this->use_post_based_groups ? 100 : 0),$this->admingroups)) ? 1 : 0;
		$USER_DATA['can_see_all_albums'] = $USER_DATA['has_admin_access'];

		// avoids a template error
		if (!$USER_DATA['user_id']) $USER_DATA['can_create_albums'] = 0;

		// For error checking
		$CONFIG['TABLE_USERS'] = '**ERROR**';

		define('USER_ID', $USER_DATA['user_id']);
		define('USER_NAME', addslashes($USER_DATA['user_name']));
		define('USER_GROUP', $USER_DATA['group_name']);
		define('USER_GROUP_SET', $user_group_set);
		define('USER_IS_ADMIN', $USER_DATA['has_admin_access']);
		define('USER_CAN_SEND_ECARDS', (int)$USER_DATA['can_send_ecards']);
		define('USER_CAN_RATE_PICTURES', (int)$USER_DATA['can_rate_pictures']);
		define('USER_CAN_POST_COMMENTS', (int)$USER_DATA['can_post_comments']);
		define('USER_CAN_UPLOAD_PICTURES', (int)$USER_DATA['can_upload_pictures']);
		define('USER_CAN_CREATE_ALBUMS', (int)$USER_DATA['can_create_albums']);
		define('USER_UPLOAD_FORM', (int)$USER_DATA['upload_form_config']);
		define('CUSTOMIZE_UPLOAD_FORM', (int)$USER_DATA['custom_user_upload']);
		define('NUM_FILE_BOXES', (int)$USER_DATA['num_file_upload']);
		define('NUM_URI_BOXES', (int)$USER_DATA['num_URI_upload']);
		
		// Redirect people who forgot password - todo: correct for CPG 1.5
		if (defined('FORGOT_PASSWD_PHP')) $this->redirect('/index.php?option=com_registration&task=lostPassword');
	}
	
	function logout_page(){
		$this->_mainframe->logout();
	}
	
	function login_page() {}
	
	function view_users() {}
	
	function view_profile() {}
	
	function login($username, $password, $remember){
	
		global $USER_DATA;

		JRequest::setVar( 'remember' , $remember ? 'yes' : 'no');
		$this->_mainframe->login($username, $password);
		$this->_user =& $this->_mainframe->getUser();
		
		$USER_DATA['user_id'] = $this->_user->get('id');
		$USER_DATA['user_name'] = $this->_user->get('name');
			
		return $USER_DATA['user_id'] ? $USER_DATA : false;
	}
	
	function get_users($options = array())
    {
    	global $CONFIG;
		
		// Copy UDB fields and config variables (just to make it easier to read)
    	$f =& $this->field;
		$C =& $CONFIG;
		
		// Sort codes 
        $sort_codes = array('name_a' => 'name ASC',
                            'name_d' => 'user_name DESC',
                            'group_a' => 'group_name ASC',
                            'group_d' => 'group_name DESC',
                            'reg_a' => 'user_regdate ASC',
                            'reg_d' => 'user_regdate DESC',
                            'pic_a' => 'pic_count ASC',
                            'pic_d' => 'pic_count DESC',
                            'disku_a' => 'disk_usage ASC',
                            'disku_d' => 'disk_usage DESC',
                            'lv_a' => 'user_lastvisit ASC',
                            'lv_d' => 'user_lastvisit DESC',
                           );
        
		$sql = "SELECT group_id, group_name, group_quota FROM {$C['TABLE_USERGROUPS']}";

		$result = cpg_db_query($sql);
		
		$groups = $quotas = array();
	
		while ($row = mysql_fetch_assoc($result)) {
			$groups[$row['group_id']] = $row['group_name'];
			$quotas[$row['group_id']] = $row['group_quota'];
		}
		
		if (in_array($options['sort'], array('group_a', 'group_d', 'pic_a', 'pic_d', 'disku_a', 'disku_d'))){
			$sort = '';
			list($this->sortfield, $this->sortdir) = explode(' ', $sort_codes[$options['sort']]);
			$this->adv_sort = true;
			
		} else {
			$sort = "ORDER BY " . $sort_codes[$options['sort']];
			$this->adv_sort = false;
		}

		// Build WHERE clause, if this is a username search
        if ($options['search']) {
            $options['search'] = 'AND u.'.$f['username'].' LIKE "'.$options['search'].'" ';
        }
		
		// Main array to hold our user data
		$userlist = array();
		
		// These sorting methods need the cpg pics table, do that first
		if (in_array($options['sort'], array('pic_a', 'pic_d', 'disku_a', 'disku_d'))){
			
			$sql = "SELECT owner_id, COUNT(pid) as pic_count, ROUND(SUM(total_filesize)/1024) as disk_usage FROM {$C['TABLE_PICTURES']} WHERE owner_id <> 0 GROUP BY owner_id ORDER BY {$sort_codes[$options['sort']]} LIMIT {$options['lower_limit']}, {$options['users_per_page']}";
			$result = cpg_db_query($sql);
			
			// If no records, return empty value
			if (!mysql_num_rows($result)) {
				return array();
			}
		
			while ($row = mysql_fetch_assoc($result)) $userlist[$row['owner_id']] = $row;
			mysql_free_result($result);

			$user_list_string = implode(', ', array_keys($userlist));

			$sql = "SELECT u.{$f['user_id']} as user_id, {$f['username']} as user_name, {$f['email']} as user_email, {$f['regdate']} as user_regdate, {$f['lastvisit']} as user_lastvisit, 0 as pic_count, 0 as disk_usage, {$f['active']} as user_active ".
               "FROM {$this->usertable} AS u ".
               "WHERE u.{$f['user_id']} IN ($user_list_string) GROUP BY u.{$f['user_id']}";
		
			$result = cpg_db_query($sql, $this->link_id);
		
			// If no records, return empty value
			if (!mysql_num_rows($result)) {
				return array();
			}
		
			while ($row = mysql_fetch_assoc($result)) $userlist[$row['user_id']] = array_merge($userlist[$row['user_id']], $row);
			mysql_free_result($result);

		} else {
		
			$sql = "SELECT u.{$f['user_id']} as user_id, {$f['username']} as user_name, {$f['email']} as user_email, {$f['regdate']} as user_regdate, {$f['lastvisit']} as user_lastvisit, 0 as pic_count, 0 as disk_usage, {$f['active']} as user_active ".
               "FROM {$this->usertable} AS u ".
               "WHERE u.{$f['user_id']} > 0 " . $options['search'].
               "GROUP BY u.{$f['user_id']} " . $sort . 
			   " LIMIT {$options['lower_limit']}, {$options['users_per_page']}";
		
			$result = cpg_db_query($sql, $this->link_id);
		
			// If no records, return empty value
			if (!mysql_num_rows($result)) {
				return array();
			}
		
			while ($row = mysql_fetch_assoc($result)) $userlist[$row['user_id']] = $row;
			mysql_free_result($result);
			
			$user_list_string = implode(', ', array_keys($userlist));
		
			$sql = "SELECT owner_id, COUNT(pid) as pic_count, ROUND(SUM(total_filesize)/1024) as disk_usage FROM {$C['TABLE_PICTURES']} WHERE owner_id IN ($user_list_string) GROUP BY owner_id";

			$result = cpg_db_query($sql);

			while ($owner = mysql_fetch_assoc($result)) $userlist[$owner['owner_id']] = array_merge($userlist[$owner['owner_id']], $owner);
		}
		
		foreach ($userlist as $uid => $user){
			
			$grps = array();
		
			if ($this->use_post_based_groups){
	
				$sql = "SELECT u.{$f['usertbl_group_id']}+100 AS group_id FROM 
					{$this->usertable} AS u
					WHERE u.{$f['user_id']} = $uid";
	
				$result = cpg_db_query($sql, $this->link_id);
				$row = mysql_fetch_assoc($result);
	
				$gid = $row['group_id'];
				
			} else {
				$gid = in_array($row[$this->field['usertbl_group_id']] , $this->admingroups) ? 1 : 2;
			}
			
			$userlist[$uid]['group_name'] = $groups[$gid]; 
			$userlist[$uid]['group_quota'] = $quotas[$gid];
		}

		if ($this->adv_sort) usort($userlist, array('core_udb', 'adv_sort'));

        return $userlist;
    }
    
    function collect_groups(){
    
		$sql = "SELECT * FROM {$this->groupstable}";

		$result = cpg_db_query($sql, $this->link_id);

		$udb_groups = array();

		while ($row = mysql_fetch_assoc($result)){
			$udb_groups[$row[$this->field['grouptbl_group_id']]+100] = $row[$this->field['grouptbl_group_name']];
		}
		
		// Patch in a group for guests
		$udb_groups[$this->guestgroup] = 'Guests';
    
    	return $udb_groups;    
	}
	
	// Edit user profile
	function edit_profile($uid)
	{
		$this->redirect($this->page['edituserprofile']);
	}
}

// and go !
$cpg_udb = new cpg_udb;